// Gas.vert

#version 120
#extension GL_EXT_gpu_shader4 : enable

uniform int ensembleStep;
uniform int lightNumber;
uniform float scatter;
uniform float ambientFactor;

varying vec3 position;
varying vec3 normal;
varying vec4 diffuseAndAmbient;

varying vec2 ensembleParamA;
varying float ensembleParamB;

float GetSpotEffect(int i, float spotAngleCos){
    float spotCosCutoff = gl_LightSource[i].spotCosCutoff;
    float spotEffect = 0.0;

    if (spotCosCutoff < -0.99){
	spotEffect = 1.0;
    }
    else if (spotAngleCos > spotCosCutoff) {
    	spotEffect = 1.0;//pow(spotEffect, gl_LightSource[i].spotExponent);
    }
    else if (spotAngleCos > spotCosCutoff * 0.9) {
	float factor = (spotAngleCos - spotCosCutoff * 0.9) / spotCosCutoff / 0.1;
	spotEffect = -2.0*factor*factor*factor + 3.0*factor*factor;
    }
    else
	spotEffect = 0.0;

    return spotEffect;
}

void main(void)
{
    gl_TexCoord[0] = gl_MultiTexCoord0;
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;

    gl_FrontColor = gl_Color;

    position = vec3(gl_ModelViewMatrix * gl_Vertex);
    normal = gl_NormalMatrix * gl_Normal;

    float diffuseTotal = 0.0;
    for (int i = 0; i < lightNumber; i++){
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float strength = gl_LightSource[i].diffuse.r;
	float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				 -lightVec);
	float spotEffect = GetSpotEffect(i, spotAngleCos);

	// W 
	float attenuation = 1.0 / (gl_LightSource[i].constantAttenuation +
				   gl_LightSource[i].linearAttenuation * dis +
				   gl_LightSource[i].quadraticAttenuation * dis * dis);
	
	vec3 viewVec = normalize(-position);
	// fBt[Y 
	float diffuse = dot(lightVec, viewVec);

	diffuse = diffuse * 1.0 * scatter + (1.0 - ambientFactor);
	diffuse *= spotEffect * attenuation * strength;
	
	diffuseTotal += diffuse;
    }

    float ambient = gl_LightSource[0].ambient.r;
    
    diffuseAndAmbient= vec4(diffuseTotal + ambient*ambientFactor,
			    diffuseTotal + ambient*ambientFactor,
			    diffuseTotal + ambient*ambientFactor, 1);
    
    ensembleParamA = vec2(1,0.5)*ensembleStep;
    ensembleParamB = 43758.5453+ensembleStep;
}
